"""
Setup script for DeltaFQ.
This file is kept for compatibility but configuration is now in pyproject.toml.
"""

from setuptools import setup

# All configuration is now in pyproject.toml
# This setup() call is minimal and just triggers setuptools to read from pyproject.toml
setup()
